{--------------------------------------------------------}
{                                                        }
{    Grafivox - gerador de grficos para cegos           }
{                                                        }
{    Mdulo de controle da impresso braille             }
{                                                        }
{    Autor:  Jos Antonio Borges                         }
{                                                        }
{    Em agosto/2018                                      }
{                                                        }
{--------------------------------------------------------}

unit gfbraille;

interface

uses
    dvcrt, dvwin, dvform,
    windows, messages, graphics, dialogs, sysutils, WinSpool,
    gfvars, gfmsg, gfArq, gfConfPrn;

function AMBCodeParaPontos (c: char): byte;
procedure imprimeBraille (demo: boolean);

const
   PerkinsParaMetaBraille: array [0..63] of char = (
   {0}   ' ',  'a',  ',',  'b', '''',  'k',  ';',  'l',
         '^',  'c',  'i',  'f', '',   'm',  's',  'p',
   {1}   '~', 'e', ':', 'h', '*', 'o', '!', 'r',
         '', 'd', 'j', 'g', '', 'n', 't', 'q',
   {2}   '', '', '?', '', '-', 'u', '"', 'v',
         '{', '', '', '', '', 'x', '', '',
   {3}   '$', '', '', '', '}', 'z', '=', '',
         '_', '', 'w', '', '#', 'y', '', '' );

    MetaBrailleParaAMBCode: array [#0..#255] of byte = (
      {00}  $00,$00,$00,$00, $00,$00,$00,$00, $20,$00,$00,$00, $00,$00,$00,$2E,
      {10}  $00,$00,$00,$00, $00,$00,$00,$00, $00,$00,$00,$00, $00,$00,$00,$00,
      {20}  $20,$36,$38,$23, $3B,$25,$26,$27, $3C,$3E,$39,$36, $31,$2D,$27,$34,
      {30}  $6A,$61,$62,$63, $64,$65,$66,$67, $68,$69,$33,$32, $7B,$37,$6F,$35,
      {40}  $3A,$41,$42,$43, $44,$45,$46,$47, $48,$49,$4A,$4B, $4C,$4D,$4E,$4F,
      {50}  $50,$51,$52,$53, $54,$55,$56,$57, $58,$59,$5A,$28, $2C,$29,$60,$5F,
      {60}  $2C,$61,$62,$63, $64,$65,$66,$67, $68,$69,$6A,$6B, $6C,$6D,$6E,$6F,
      {70}  $70,$71,$72,$73, $74,$75,$76,$77, $78,$79,$7A,$2E, $5F,$30,$22,$7F,
      {80}  $40,$5C,$3D,$2A, $3E,$24,$86,$26, $3C,$89,$21,$7D, $8C,$25,$8E,$8F,
      {90}  $90,$38,$38,$3F, $5B,$27,$96,$3A, $98,$99,$9A,$9B, $9C,$9D,$9E,$9F,
      {A0}  $28,$36,$2B,$29, $A4,$A5,$5F,$A7, $20,$A9,$61,$38, $5E,$AD,$AE,$AF,
      {B0}  $30,$B1,$62,$63, $39,$B5,$B6,$B7, $B8,$61,$4F,$38, $BC,$BD,$BE,$35,
      {C0}  $24,$28,$2A,$33, $3E,$2A,$3E,$26, $21,$3D,$3C,$24, $2F,$27,$25,$5D,
      {D0}  $D0,$7D,$2B,$3F, $5B,$5B,$D6,$38, $D8,$29,$29,$3A, $5C,$DD,$DE,$DF,
      {E0}  $24,$28,$2A,$3E, $3E,$2A,$3E,$26, $21,$3D,$3C,$24, $2F,$2F,$25,$7D,
      {F0}  $F0,$7D,$2B,$2B, $3F,$5B,$5B,$34, $F8,$29,$29,$3A, $5C,$2C,$FE,$34);

const
    americanBrailleTable: array [#0..#255] of byte = (
      {00}  $C8,$C1,$C3,$C9, $D9,$D1,$CB,$DB, $D3,$CA,$DA,$C5, $C7,$CD,$DD,$D5,
      {10}  $CF,$DF,$D7,$CE, $DE,$E5,$E7,$FA, $ED,$FD,$F5,$EA, $F3,$FB,$D8,$F8,
      {20}  $00,$2E,$10,$3C, $2B,$29,$2F,$04, $37,$3E,$21,$2C, $20,$24,$28,$0C,
      {30}  $34,$02,$06,$12, $32,$22,$16,$36, $26,$14,$31,$30, $23,$3F,$1C,$39,
      {40}  $48,$41,$43,$49, $59,$51,$4B,$5B, $53,$4A,$5A,$45, $47,$4D,$5D,$55,
      {50}  $4F,$5F,$57,$4E, $5E,$65,$67,$7A, $6D,$7D,$75,$2A, $33,$7B,$58,$38,
      {60}  $08,$01,$03,$09, $19,$11,$0B,$1B, $13,$0A,$1A,$05, $07,$0D,$1D,$15,
      {70}  $0F,$1F,$17,$0E, $1E,$25,$27,$3A, $2D,$3D,$35,$2A, $33,$3B,$18,$78,
      {80}  $88,$81,$83,$89, $99,$91,$8B,$9B, $93,$8A,$9A,$85, $87,$8D,$9D,$95,
      {90}  $8F,$9F,$97,$8E, $9E,$A5,$A7,$BA, $AD,$BD,$B5,$AA, $B3,$BB,$98,$B8,
      {A0}  $C0,$6E,$6B,$7C, $69,$6F,$71,$54, $50,$76,$80,$63, $72,$64,$66,$62,
      {B0}  $74,$6C,$46,$52, $44,$70,$56,$68, $60,$42,$40,$5C, $77,$7F,$7E,$79,
      {C0}  $D6,$E1,$C2,$D0, $EF,$DC,$C4,$EC, $F6,$E3,$C6,$FF, $E6,$E9,$D2,$F7,
      {D0}  $E0,$E8,$D4,$F9, $F2,$F0,$EE,$61, $CC,$F4,$F1,$E2, $FE,$EB,$E4,$FC,
      {E0}  $96,$A1,$82,$90, $AF,$9C,$84,$AC, $B6,$A3,$86,$BF, $A6,$A9,$92,$B7,
      {F0}  $A0,$A8,$94,$B9, $B2,$B0,$AE,$4C, $8C,$B4,$B1,$A2, $BE,$AB,$A4,$BC);

(*
    tabPontosBrl: array [#$20..#$7f] of string[6] = (
        '000000','011101','000010','001111',  // 20
        '110101','100101','111101','001000',
        '111011','011111','100001','001101',
        '000001','001001','000101','001100',
        '001011','010000','011000','010010',  // 30
        '010011','010001','011010','011011',
        '011001','001010','100011','000011',
        '110001','111111','001110','100111',
        '000100','100000','110000','100100',  // 40
        '100110','100010','110100','110110',
        '110010','010100','010110','101000',
        '111000','101100','101110','101010',
        '111100','111110','111010','011100',  // 50
        '011110','101001','111001','010111',
        '101101','101111','101011','010101',
        '110011','110111','000110','000111',
        '000100','100000','110000','100100',  // 60
        '100110','100010','110100','110110',
        '110010','010100','010110','101000',
        '111000','101100','101110','101010',
        '111100','111110','111010','011100',  // 70
        '011110','101001','111001','010111',
        '101101','101111','101011','010101',
        '110011','110111','000110','111111'
    );
*)

implementation

uses
    gfdesen;

const
    LF        = #$0A;       { Line Feed }
    CR        = #$0D;       { Carriage Return }
    CRLF      = CR+LF;      { Carriage Return + Line Feed }
    FORMFEED  = #$0c;       { Form Feed }

    INDEX_INIGRAF = ESC+'1';
    INDEX_FIMGRAF = ESC+']';

function AMBCodeParaPontos (c: char): byte;
begin
    result := americanBrailleTable[c] and $3f;
end;

function colorToByte (x: TColor): byte;
begin
    if x <> ClWhite then
        result := 1
    else
        result := 0;
end;

function imprimeIndex: boolean;
var b: byte;
    x, y, ym: integer;
    xx, yy: integer;
    linha: string;
    ok: boolean;
label
    fim,
    pula;
begin
  ok := escreveDispositivo (INDEX_INIGRAF);       { ESC + '1' }
  if not ok then
      goto fim;

  ym := 1;
  for y := FGrafico.Height-1 downto 0 do
      begin
          for x := 0 to (xMaxBraille-1) do
              if FGrafico.Canvas.Pixels[x,y] <> clWhite then
                  begin
                      ym := y;
                      goto pula;
                  end;
      end;

pula:
  for y := 0 to (ym div 3) do
      begin
          linha := '';
          yy := y*3;
          for x := 0 to (FGrafico.Width-1) div 2 do
              with FGrafico.Canvas do
                  begin
                      xx := x*2;
                      b := colorToByte (Pixels[xx+0, yy+0]) shl 0 +
                           colorToByte (Pixels[xx+0, yy+1]) shl 1 +
                           colorToByte (Pixels[xx+0, yy+2]) shl 2 +
                           colorToByte (Pixels[xx+1, yy+0]) shl 3 +
                           colorToByte (Pixels[xx+1, yy+1]) shl 4 +
                           colorToByte (Pixels[xx+1, yy+2]) shl 5;
                      linha := linha + chr(MetaBrailleParaAMBCode[PerkinsParaMetaBraille[b]]);
                  end;

          ok := ok and escreveDispositivo (trimRight(linha) + CRLF);
      end;

    ok := ok and escreveDispositivo (INDEX_FIMGRAF);  { ESC + ']' }
    ok := ok and escreveDispositivo (FORMFEED);
    fechaImpressora;

fim:
    if not ok then
        mensagem ('GFERRDSP', 2);  {'Erro no dispositivo'}
    result := ok;
end;

{--------------------------------------------------------}
{                     imprime em braille                 }
{--------------------------------------------------------}

procedure imprimeBraille (demo: boolean);
var
    nomeImpressora: string;
begin
    mensagem ('GFSBRL', 0); {'Selecione a impressora Braille'}

    nomeImpressora := escolheImprPadrao;
    if nomeImpressora = '' then
        exit;

    writeln;
    mensagem ('GFIMPREM', 0);   {'Imprimindo em '}
    sintWriteln (nomeImpressora);
    if not alocaImpressora (nomeImpressora) then
        begin
            mensagem ('GFPROBLI', 2);  {'Problemas na impressora'}
            exit;
        end;

    mensagem ('GFIMPROK', 2); {'Impressora ok. Iniciando impresso.'}

    saidaGrafica := paraBraille;
    criaBitmapParaDesenho;
    if demo then
        produzOGraficoDemo
    else
        produzOGrafico;

    imprimeIndex;
    FGrafico.Free;

    clrscr;
    gfmsg.tituloJanela(nomeArq);
    gotoxy (1, 3);
end;

end.

