{--------------------------------------------------------}
{                                                        }
{    Grafivox - gerador de grficos para cegos           }
{                                                        }
{    Mdulo de desenho visual                            }
{                                                        }
{    Autor:  Jos Antonio Borges                         }
{                                                        }
{    Em agosto/2018                                      }
{                                                        }
{--------------------------------------------------------}

unit gfdesen;

interface
uses
     windows, graphics, classes,
     dvcrt, dvwin, dvform, dvArq, sysutils,
     JPeg, PNGImage, gfvars, gfArq, gfmsg, gfTexto,
     gfEquacao, gfGeraBrl, gfBraille, gfBrUnif, gfDados;

procedure visualiza (demo: boolean);
procedure gravaArquivoGrafico;
procedure criaBitmapParaDesenho;
procedure produzOGraficoDemo;
procedure produzOGrafico;

implementation

uses
    Math, Types;

const
    epsilon = 1e-32;

type
    TVet = array [0..20] of real;

var
    dxTexto, dyTexto: integer;

    xim, yim, xfm, yfm: real;
    xit, yit, xft, yft: integer;

    escalax, escalay: real;
    xo, yo: real;
var
    np: integer;
    xp, fx: TVet;

    ultx, ulty: real;

    erroDetectado: boolean;
    linProc: integer;   { Linha atualmente processada }

{--------------------------------------------------------}
{            transformao de mundo para tela            }
{--------------------------------------------------------}

procedure WVprep;
begin
    escalax := (xft-xit) / (xfm-xim);
    escalay := (yft-yit) / (yfm-yim);
    xo := xit-(xim*escalax);
    yo := yit-(yim*escalay);
end;

function W2VX(x: real): integer;
begin
    result := trunc(xo + x*escalax);
end;

function W2VY(y: real): integer;
begin
    result := trunc(yo + y*escalay);
end;

{-----------------------------------------------------------}
{               Configura domnio (mundo)                   }
{-----------------------------------------------------------}

procedure DefineDominio (xi, yi, xf, yf: real);
begin
    with FGrafico.Canvas do
        begin
            xim := xi;   yim := yi;
            xfm := xf;   yfm := yf;
        end;
    wvprep;
end;

{-----------------------------------------------------------}
{                   Configura viewport                      }
{-----------------------------------------------------------}

procedure DefineViewport (xji, yji, xjf, yjf: real);
begin
    xit := trunc(xji * FGrafico.width);
    yit := trunc(yji * FGrafico.height);
    xft := trunc(xjf * FGrafico.width);
    yft := trunc(yjf * FGrafico.height);
    wvprep;
end;

{-----------------------------------------------------------}
{          Configura window e Viewport default              }
{-----------------------------------------------------------}

procedure criaWVDefault;
begin
    defineDominio (-20, -15, 20, 15);
    
    // y cresce para baixo na maioria dos dispositivos
    defineViewport (0, 1, 1, 0);
end;

{--------------------------------------------------------}
{        rotinas bsicas usando coordenadas do mundo     }
{--------------------------------------------------------}

procedure desenhaPonto (xm, ym: real);
var xt, yt: integer;
begin
    with FGrafico.canvas do
        begin
            xt := W2VX(xm);
            yt := W2VY(ym);
            if saidaGrafica = paraBraille then
                pixels[xt, yt] := 1
            else
                begin
                  moveTo (xt-1, yt-1);
                   lineTo (xt-1, yt+1);
                   lineTo (xt+1, yt+1);
                   lineTo (xt+1, yt-1);
                   lineTo (xt-1, yt-1);
                end;
        end
end;

{--------------------------------------------------------}

procedure desenhaReta (xm0, ym0, xm1, ym1: real);
begin
    with FGrafico.canvas do
        begin
            moveTo (W2VX(xm0), W2VY(ym0));
            lineTo (W2VX(xm1), W2VY(ym1));
        end;
end;

{--------------------------------------------------------}

procedure desenhaPoligono (npt: integer; vx, vy: TVet; fechado: boolean);
var i: integer;
begin
    for i := 0 to npt-2 do
        desenhaReta (vx[i], vy[i], vx[i+1], vy[i+1]);
    if fechado then
        desenhaReta (vx[npt-1], vy[npt-1], vx[0], vy[0]);
end;

{--------------------------------------------------------}

procedure desenhaSeta (xm0, ym0, xm1, ym1: real);

var
   compSeta, beta: real;
   h: real;
   xp, yp: integer;
   xs, ys: real;
   xs1, ys1: integer;
   xs2, ys2: integer;

   x0, y0, x1, y1: integer;

begin
    x0 := W2VX(xm0);
    y0 := W2VY(ym0);
    x1 := W2VX(xm1);
    y1 := W2VY(ym1);

    desenhaReta (xm0, ym0, xm1, ym1);
    xp := x1-x0;
    yp := y1-y0;

    h := sqrt (xp * xp + yp * yp);
    if h < 1 then h := 1;

    if saidaGrafica <> paraBraille then
        begin
            compSeta := 15;
            beta := 20 * PI / 180;
        end
    else
        begin
            compSeta := 5;
            beta := 30 * PI / 180;
        end;

    xs := (compSeta * xp) / h;
    ys := (compSeta * yp) / h;

    xs1 := trunc(xs * cos(beta)  - ys * sin(beta));
    ys1 := trunc(xs * sin(beta)  + ys * cos(beta));
    xs2 := trunc(xs * cos(-beta) - ys * sin(-beta));
    ys2 := trunc(xs * sin(-beta) + ys * cos(-beta));

    with FGrafico.canvas do
        begin
            Polygon([Point(x1, y1),
                     Point(x0+xp-xs1, y0+yp-ys1),
                     Point(x0+xp-xs2, y0+yp-ys2)]);
        end;
end;

{--------------------------------------------------------}

procedure desenhaRetangulo (xm0, ym0, dxm, dym: real);
var
    xm1, ym1: real;
begin
    xm1 := xm0 + dxm;
    ym1 := ym0 + dym;
    if (dxm = 0) and (dym = 0) then
        FGrafico.Canvas.Pixels[W2VX(xm0),W2VY(ym0)] := FGrafico.Canvas.Pen.Color
    else
        FGrafico.Canvas.Rectangle(W2VX(xm0), W2VY(ym0), W2VX(xm1), W2VY(ym1));
end;

{--------------------------------------------------------}

procedure desenhaCirc (xc, yc, r: real);
var
   x, y, ang: real;
begin
   ang := 0;
   while ang <= 2*pi do
       begin
            x := xc + r*cos(ang);
            y := yc + r*sin(ang);
            if ang = 0 then
                FGrafico.canvas.moveTo (W2VX(x), W2VY(y))
            else
                FGrafico.canvas.lineTo (W2VX(x), W2VY(y));
            ang := ang + 0.05;
       end;
end;

{--------------------------------------------------------}

procedure desenhaElipse (xc, yc, a, b: real);
var
   x, y, ang: real;
begin
   ang := 0;
   with FGrafico.canvas do
     while ang <= 2*pi do
       begin
            x := xc + a*cos(ang);
            y := yc + b*sin(ang);
            if ang = 0 then
                moveTo (W2VX(x), W2VY(y))
            else
                lineTo (W2VX(x), W2VY(y));
            ang := ang + 0.05;
       end;
end;

{--------------------------------------------------------}

procedure desenhaPolinomio (grau: integer; coefRev: TVet; xmin, xmax: real);
var x, y, xx: real;
    g: integer;
const epsilon = 0.00001;
begin
    x := xmin;
    while x <= xmax do
        begin
            xx := 1;
            y := 0;
            for g := 0 to grau do
                begin
                    y := y + xx * coefRev[grau-g];
                    xx := xx * x;
                end;
            if x = xmin then
                FGrafico.canvas.moveTo (W2VX(x), W2VY(y))
            else
                FGrafico.canvas.lineTo (W2VX(x), W2VY(y));
            x := x + 0.01;
        end;
end;

{--------------------------------------------------------}

procedure desenhaLagrange (npt: integer; vx, vy: TVet);

        function Lagrange(n, k: integer; val: real): real;
        var
            res: real;
            i: integer;
        begin
            res := 1;
            for i := 0 to n-1 do
                if (i <> k) and ((xp[k] - xp[i]) <> 0) then
                    res := res * ((val - xp[i]) / (xp[k] - xp[i]));
            result := res;
        end;

        function Calculate(n: integer; x: real): real;
        var
            intPolasyon: real;
            i: integer;
            L: TVet;
        begin
            intPolasyon := 0;
            for i := 0 to n-1 do
                begin
                    L[i] := Lagrange(n, i, x);
                    intPolasyon := intPolasyon + (fx[i] * L[i]);
                end;
            result := intPolasyon;
        end;

        procedure ordena (np: integer);
        var i, j: integer;
            troca: real;
        begin
            for i := 0 to np-2 do
                for j := i to np-1 do
                    if xp[i] > xp[j] then
                        begin
                            troca := xp[i];  xp[i] := xp[j];  xp[j] := troca;
                            troca := fx[i];  fx[i] := fx[j];  fx[j] := troca;
                        end;
        end;

var
    x, calc: real;
begin
    xp := vx;
    fx := vy;
    np := npt;

    ordena(np);

    x := xp[0];
    while x <= xp[np-1] do
        begin
            calc := Calculate(np, x);

            if x = xp[0] then
                FGrafico.canvas.moveto (W2VX(x), W2VY(calc))
            else
                FGrafico.canvas.lineto (W2VX(x), W2VY(calc));

            x := x + 0.05;
        end;
end;

{--------------------------------------------------------}

procedure desenhaTexto (x, y: real; s: string);
begin
    with FGrafico.Canvas do
        begin
            Brush.Color := CLWHITE;
            Brush.Style := bsClear;
            TextOut(W2VX(x), W2VY(y)-(dytexto*13 div 16), s);
        end;
end;

{--------------------------------------------------------}

procedure desenhaTextoCentrado (x, y: real; s: string);
begin
    with FGrafico.Canvas do
        begin
            Brush.Color := CLWHITE;
            Brush.Style := bsClear;
            TextOut(W2VX(x)-TextWidth(s) div 2,
                    W2VY(y)-(dytexto*13 div 16), s);
        end;
end;

{--------------------------------------------------------}

procedure desenhaHistograma(xi, yi, largura: real;
                            nBarras: integer; altura: TVet);
var i: integer;

begin
    for i := 0 to nBarras-1 do
        begin
            FGrafico.Canvas.Rectangle(W2VX(xi), W2VY(yi),
                                      W2VX(xi+largura), W2VY(yi+altura[i]));
            xi := xi + largura;
        end;
end;

{--------------------------------------------------------}
{        Cria Bitmap em Arquivo BMP para Desenho         }
{--------------------------------------------------------}

procedure criaBitmapParaDesenho;
var
    lpRect: TRect;

begin
    case saidaGrafica of
        paraBraille:    with lpRect do
                            begin
                                Left := 0;  Right  := xMaxBraille;
                                Top  := 0;  Bottom := yMaxBraille;
                            end;
        paraTinta:  with lpRect do
                        begin
                            Left := 0;  Right  := xMaxPrinter;
                            Top  := 0;  Bottom := yMaxPrinter;
                        end;
        paraTela:   GetClientRect(crtWindow, lpRect);
    end;

    FGrafico := TBitmap.Create;
    FGrafico.PixelFormat := pf32bit;
    FGrafico.Width  := lpRect.Right  - lpRect.Left;
    FGrafico.Height := lpRect.bottom - lpRect.top;

    with FGrafico.Canvas do
        begin
            // se tela, deixa uma linha de texto alfanumrico embaixo
            if saidaGrafica = paraTela then
                FGrafico.Height := FGrafico.Height - 42;

            Brush.Color := clWhite;
            Brush.Style := bsSolid;
            FillRect(rect(0, 0, FGrafico.Width, FGrafico.Height));

            Font.Name := 'Courier New';
            case saidaGrafica of
                paraBraille:   Font.Size := -12;
                paraTela:      Font.Size := -24;
                paraTinta:     Font.Size := -16;
            end;

            Brush.Color := clWhite;
            Brush.Style := bsClear;

            Pen.Color   := clBlack;
            Pen.Mode    := pmCopy;
            Pen.Style   := psSolid;
            Pen.Width   := 1;

            Font.Color  := clBlack;
            dxTexto := TextWidth ('G');
            dyTexto := TextHeight ('');
        end;
end;

{--------------------------------------------------------}
{               desenha eixos de demonstrao            }
{--------------------------------------------------------}

procedure desenhaEixosDemo;
var
    ix, iy: integer;
begin
    with FGrafico.canvas do
        begin
            moveto (W2VX(-10.0), W2VY(0));
            lineto (W2VX(10.0), W2VY(0));

            for ix := -10 to 10 do
                begin
                    if saidaGrafica = paraBraille then
                        moveTo (W2VX(ix), W2VY(0)+2)
                    else
                        moveTo (W2VX(ix), W2VY(0)+10);
                    lineTo (W2VX(ix), W2VY(0));
                end;

            moveto (W2VX(0), W2VY(-10.0));
            lineto (W2VX(0), W2VY(10.0));

            for iy := -10 to 10 do
                begin
                    if saidaGrafica = paraBraille then
                        moveTo (W2VX(0)-2,  W2VY(iy))
                    else
                        moveTo (W2VX(0)-10, W2VY(iy));
                    lineTo (W2VX(0),    W2VY(iy));
                end;

            for ix := -10 to 10 do
                for iy := -10 to 10 do
                        desenhaPonto (ix, iy);
        end;
end;

{--------------------------------------------------------}
{              produz o grfico (demonstrao)           }
{--------------------------------------------------------}

procedure produzOGraficoDemo;
var
    vx, vy, coefr: TVet;

begin
    with FGrafico.Canvas do
        begin
              Rectangle (0, 0, FGrafico.width, FGrafico.height);

              criaWVDefault;
              desenhaEixosDemo;

              pen.Color := clMaroon;
              desenhaReta (0,1, 0,5);
              desenhaReta (1,1, 1,5);
              desenhaReta (2,1, 2,5);
              desenhaReta (7.0, -1.0, 6.0, 3.0);

              pen.Color := clBlack;
              desenhaSeta (6.0, -2.0, 5.0, 4.0);
              desenhaSeta (5.0, 4.0, 8.0, 10.0);

              pen.Color := clGreen;
              brush.color := clYellow;
              brush.style := bsCross;
              pen.Width := 4;
              desenhaRetangulo (-5, -3, 4, 3);
              pen.Width := 1;

              pen.Color := clBlack;
              brush.style := bsFDiagonal;
              desenhaCirc (-5, -6, 2);
              desenhaElipse (-5, -6, 3, 0.5);

              pen.Color := clRed;
              coefr[0] := 1;
              coefr[1] := -2;
              coefr[2] := -1;
              desenhaPolinomio (2, coefr, -2, 4);

              vx[0] := 1;   vy[0] := -5;
              vx[1] := 3;   vy[1] := -2;
              vx[2] := 5;   vy[2] := -4;
              vx[3] := 7;   vy[3] := -4;
              vx[4] := 9;   vy[4] := -6;
              desenhaPonto (0, 0);
              desenhaPoligono (5, vx, vy, true);
              pen.Color := clBlue;
              desenhaLagrange(5, vx, vy);

              font.Color := clBlack;
              DesenhaTexto (-19.6, 9, 'Curso de Fsica I');
              DesenhaTexto (-19.6, 8, 'Testes do Grafivox');
              DesenhaTextoCentrado (0, -9.5, 'UFRJ - 2023');

        end;
end;

{--------------------------------------------------------}
{              processamento dos comandos                }
{--------------------------------------------------------}

var
    sLinha:    string;
    sCmd:      string;

function pegaToken: string;
var p: integer;
begin
    result := '';
    sLinha := trim(sLinha);
    if sLinha = '' then
        exit;
    p := pos (' ', sLinha);
    if p > 0 then
    begin
        result := copy (sLinha, 1, p-1);
        delete (sLinha, 1, p);
    end
    else
        begin
            result := sLinha;
            slinha := '';
        end;
end;

function pegaNome (var s: string): boolean;
begin
    s := pegaToken;
    result := s <> '';
end;

procedure mostraErro;
begin
    sintBip; sintBip; sintBip;
    mensagem ('GF_ERRL', 0);   {'Erro na linha '}
    sintWriteint (linproc);
    writeln;
    sintWriteln (listaComandos[linProc]);
    readkey;
end;

function pegaReal (var r: real): boolean;
var s: string;
begin
    s := pegaToken;
    if s = '' then
        result := False
    else
        try
            result := True;
            if decimalSeparator = '.' then
                r := strToFloat (stringReplace (s, ',', '.', []))
            else
                r := strToFloat (stringReplace (s, '.', ',', []));
        except
            erroDetectado := true;
            result := False;
        end;
end;

function pegaInteiro (var i: integer): boolean;
var s: string;
begin
    s := pegaToken;
    i := 0;
    if s = '' then
        result := False
    else
        try
            result := True;
            i := strToInt (s);
        except
            { EConvertError exception }
            result := False;
        end;
end;

function pegaPontoR2 (var x, y: real): boolean;
var relativo, polar: boolean;
begin
    result := False;

    slinha := trim (slinha);
    if slinha = '' then slinha := ' ';

    if slinha[1] = '@' then
        begin
            delete (slinha, 1, 1);
            insert ('R 0 0 ', slinha, 1);
        end;

    relativo := upcase(slinha[1]) = 'R';
    polar := upcase(slinha[1]) = 'P';
    if relativo or polar then
        delete (slinha, 1, 1);

    if not pegaReal (x) then exit;
    if not pegaReal (y) then exit;

    if relativo then
        begin
            x := ultx + x;
            y := ulty + y;
        end
    else
    if polar then
        begin
            x := ultx + x * cos(y*pi/180);
            y := ulty + x * sin(y*pi/180);
        end;
            
    ultx := x;
    ulty := y;

    result := True;
end;

procedure trataDominio;
var
    x1, y1, x2, y2: real;
begin
    if not (pegaPontoR2 (x1, x2) and pegaPontoR2 (y1, y2)) then
        exit;
    defineDominio (x1, y1, x2, y2);
end;

procedure trataContraDominio;
var
    x1, y1, x2, y2: real;
begin
    if not (pegaPontoR2 (x1, x2) and pegaPontoR2 (y1, y2)) then
        exit;
    defineViewport (x1, y2, x2, y1);  // mundo de cabea para baixo
end;

procedure trataGrade;
var
    x1, y1, x2, y2: real;
    dx, dy: real;
    x, y: real;
begin
    if not (pegaPontoR2 (x1, y1) and pegaPontoR2 (x2, y2) and
            pegaPontoR2 (dx, dy)) then
        exit;

    if dx <= 0 then exit;
    if dy <= 0 then exit;

    x := x1;
    while x <= x2 do
        begin
            y := y1;
            while y <= y2 do
                 begin
                     desenhaPonto(x, y);
                     y := y + dy;
                 end;
            x := x + dx;
        end;
end;

procedure trataEixos;
var
    xi, yi, xf, yf: real;
    xx, yy, dist: real;
begin
    if not pegaPontoR2 (xi, yi) then exit;
    if not pegaPontoR2 (xf, yf) then exit;
    if not pegaReal (dist) then
        dist := 0;

    with FGrafico.canvas do
        begin
            moveto (W2VX(xi), W2VY(yi));
            lineto (W2VX(xf), W2VY(yf));

            if dist = 0 then exit;   { sem ticks }

            { temporariamente no faz ticks se no ortogonal }
            if yi = yf then
                begin
                    xx := xi;
                    while xx <= xf do
                        begin
                            if saidaGrafica = paraBraille then
                                moveTo (W2VX(xx), W2VY(yi)+2)
                            else
                                moveTo (W2VX(xx), W2VY(yi)+10);
                            lineTo (W2VX(xx), W2VY(yi));
                            xx := xx + dist;
                        end;
                end
            else
                begin
                    yy := yi;
                    while yy <= yf do
                        begin
                            if saidaGrafica = paraBraille then
                                moveTo (W2VX(xi)-2,  W2VY(yy))
                            else
                                moveTo (W2VX(xi)-10, W2VY(yy));
                            lineTo (W2VX(xi), W2VY(yy));
                            yy := yy + dist;
                        end;
                end;
        end;

end;

procedure trataPonto;
var
    xm, ym: real;
    i: integer;
begin
    slinha := trim (slinha);
    if copy (sLinha, 1, 1) = '*' then
        begin
            for i := 0 to numDados-1 do
                desenhaPonto (dadosx[i], dadosy[i]);
        end
    else
        begin
            if not pegaPontoR2 (xm, ym) then exit;
            desenhaPonto (xm, ym);
        end;
end;

procedure trataReta;
var
    x1m, y1m, x2m, y2m: real;
begin
    if not pegaPontoR2 (x1m, y1m) then exit;
    if not pegaPontoR2 (x2m, y2m) then exit;
    desenhaReta (x1m, y1m, x2m, y2m);
end;

procedure trataSeta;
var
    x1m, y1m, x2m, y2m: real;
begin
    if not (pegaPontoR2 (x1m, y1m) and pegaPontoR2 (x2m, y2m)) then exit;
    desenhaSeta (x1m, y1m, x2m, y2m)
end;

procedure trataRetangulo;
var
    xm, ym, dxm, dym: real;
begin
    if not (pegaPontoR2 (xm, ym) and pegaPontoR2 (dxm, dym)) then exit;
    desenhaRetangulo (xm, ym, dxm, dym);
end;

procedure trataRetangRodado;
var
    x1m, y1m, dx, dy, ang, xx, yy: real;

    procedure roda (x, y, ang: real; var xr, yr: real);
    begin
        xr := x*cos(ang) - y*sin(ang);
        yr := x*sin(ang) + y*cos(ang);
    end;

begin
    if not (pegaPontoR2 (x1m, y1m) and pegaPontoR2 (dx, dy) and
            pegaReal(ang)) then exit;
    ang := ang * 3.141592 / 180;

    with FGrafico.canvas do
        begin
            moveto (W2VX(x1m), W2VY(y1m));
            roda (dx, 0, ang, xx, yy);
            lineto (W2VX(x1m+xx), W2VY(y1m+yy));
            roda (dx, dy, ang, xx, yy);
            lineto (W2VX(x1m+xx), W2VY(y1m+yy));
            roda (0, dy, ang, xx, yy);
            lineto (W2VX(x1m+xx), W2VY(y1m+yy));
            lineto (W2VX(x1m), W2VY(y1m));
        end;
end;

procedure trataPoligono;
var i, n: integer;
    x, y: TVet;
begin
    slinha := trim (slinha);
    if copy (sLinha, 1, 1) = '*' then
        begin
            n := numDados;
            for i := 0 to n-1 do
                begin
                    if i <> 0 then
                        desenhaReta (dadosx[i-1], dadosy[i-1],
                                     dadosx[i], dadosy[i]);
                end;
        end
    else
        begin
            for i := 0 to 20 do
                begin
                    if not pegaPontoR2 (x[i], y[i]) then exit;
                    if i <> 0 then
                        desenhaReta (x[i-1], y[i-1], x[i], y[i]);
                end;
        end;

    // futuramente executar o desenho preenchido
end;

procedure trataCircunf;
var
    xC, yC: real;
    raio:   real;
begin
    if not pegaPontoR2 (xC, yC) then exit;
    if (not pegaReal (raio)) or (raio < 0) then exit;
    FGrafico.Canvas.Ellipse (W2VX(xC-raio),W2VY(yC-raio),
                           W2VX(xC+raio),W2VY(yC+raio));

    // futuramente executar o desenho preenchido
end;

procedure trataElipse;
var
    xc, yc, a, b: real;
    x1, y1, x2, y2: real;
begin
    if not pegaPontoR2 (xc, yc) then exit;
    if not pegaPontoR2 (a, b) then exit;
    x1 := xc - a;
    y1 := yc - b;
    x2 := xc + a;
    y2 := yc + b;
    FGrafico.Canvas.Ellipse (W2VX(x1),W2VY(y1), W2VX(x2),W2VY(y2));

    // futuramente executar o desenho preenchido
end;

procedure trataParabola;
var
    x, y: TVet;
    temp: real;
begin
    if not pegaPontoR2 (x[0], y[0]) then exit;
    if not pegaPontoR2 (x[1], y[1]) then exit;
    if not pegaPontoR2 (x[2], y[2]) then exit;
    if x[0] > x[1] then
        begin
            temp := x[0];  x[0] := x[1];  x[1] := temp;
            temp := y[0];  y[0] := y[1];  y[1] := temp;
        end;
    if x[1] > x[2] then
        begin
            temp := x[1];  x[1] := x[2];  x[2] := temp;
            temp := y[1];  y[1] := y[2];  y[2] := temp;
        end;
    if x[1] > x[2] then
        begin
            temp := x[1];  x[1] := x[2];  x[2] := temp;
            temp := y[1];  y[1] := y[2];  y[2] := temp;
        end;

    desenhaLagrange(3, x, y);
end;

procedure trataCurva;
var i, n: integer;
    x, y: TVet;
begin
    slinha := trim (slinha);
    if copy (sLinha, 1, 1) = '*' then
        begin
            n := numDados;
            for i := 0 to n-1 do
                begin
                    x[i] := dadosx[i];
                    y[i] := dadosy[i];
                end;
        end
    else
        begin
            for i := 0 to 20 do
                begin
                    if not pegaPontoR2 (x[i], y[i]) then
                        begin
                            if i > 1 then
                                desenhaLagrange(i, x, y);
                            break;
                        end;
                end;
        end;
end;

procedure trataPolinomio;
var i, grau: integer;
    coef: TVet;
    xmin, xmax: real;
begin
    if not pegaInteiro(grau) then exit;
    for i := grau downto 0 do
        if not pegaReal (coef[i]) then exit;

    if not pegaReal (xmin) then exit;
    if not pegaReal (xmax) then exit;

    desenhaPolinomio(grau, coef, xmin, xmax);
end;

procedure trataTextoBraille;
var
    xm, ym: real;
    sTexto: string;
    xt, yt, dxt: integer;
    i: integer;
begin
    if not pegaPontoR2 (xm, ym) then
        exit;
    sTexto := TrimRight (sLinha);
    if sTexto = '' then
        exit;

    sTexto := brailleUnificado (sTexto);

    xt := W2VX(xm);
    yt := W2VY(ym);
    for i := 1 to length(sTexto) do
         begin
             dxt := produzCaracBraille (chr(MetaBrailleParaAMBCode[sTexto[i]]),
                             xt, yt);
             xt := xt + dxt;
         end;
end;

procedure trataTextoTinta;
var
    xm, ym: real;
    sTexto: string;
begin
    if saidaGrafica = paraBraille then
        begin
            trataTextoBraille;
            exit;
        end;

    if not pegaPontoR2 (xm, ym) then
        exit;
    sTexto := TrimRight (sLinha);
    if sTexto = '' then
        exit;

    escreveTexto (W2VX(xm), W2VY(ym), sTexto);
end;

procedure trataLeDados;
var nomeArq: string;
begin
    nomeArq := trim(sLinha);
    carregaDados (nomeArq);
end;

procedure trataConfigCaneta;
var largura, estilo, cor: integer;
const
    tabEstilos: array [1..3] of TPenStyle = (psSolid, psDot, psDash);
    tabCores: array [1..3] of TColor = (clBlack, clRed, clBlue);
begin
    pegaInteiro(largura);
    pegaInteiro(estilo);
    pegaInteiro(cor);
    if (largura < 1) or (largura > 5) then largura := 1;
    if not (cor in [1,2,3]) then cor := 1;
    if not (estilo in [1,2,3]) then estilo := 1;

    with FGrafico.canvas.pen do
        begin
            width := largura*3-2;
            style := tabEstilos[estilo];
            color := tabCores[cor];
        end;
end;

procedure trataConfigTextoTinta;
begin
(*
    else
        begin
            mensagem ('GFERRORT', 1);  {'Erro na orientao de texto'}
            exit;
        end;
*)
end;

procedure trataHistograma;
var xi, yi, largura: real;
    i, nBarras: integer;
    altura: TVet;
begin
    if not pegaPontoR2 (xi, yi) then
        exit;

    if not pegaReaL(largura) then exit;

    nBarras := 0;
    for i := 0 to 20 do
        begin
            if not pegaReal (altura[i]) then
                begin
                    nBarras := i - 1;
                    break;
                end;
        end;

    desenhaHistograma(xi, yi, largura, nBarras, altura);
end;

{--------------------------------------------------------}
{                     produz o grfico                   }
{--------------------------------------------------------}

procedure produzOGrafico;
var
    i: integer;

begin
    erroDetectado := false;
    criaWVDefault;
    ultx := 0;
    ulty := 0;

    for i := 0 to listaComandos.count-1 do
        begin
            linProc := i;
            sLinha := trim(listaComandos[i]);
            if (sLinha = '') or (sLinha[1] = ';') then continue;

            sCmd  := upperCase(copy (sLinha, 1, 2));
            delete (sLinha, 1, 2);

            if      sCmd = 'PT' then trataPonto
            else if sCmd = 'RT' then trataReta
            else if sCmd = 'ST' then trataSeta
            else if sCmd = 'RG' then trataRetangulo
            else if sCmd = 'RR' then trataRetangRodado
            else if sCmd = 'PO' then trataPoligono
            else if sCmd = 'CI' then trataCircunf
            else if sCmd = 'EL' then trataElipse
            else if sCmd = 'PA' then trataParabola
            else if sCmd = 'CV' then trataCurva
            else if sCmd = 'EQ' then trataEquacao

            else if sCmd = 'GR' then trataGrade
            else if sCmd = 'EX' then trataEixos

            else if sCmd = 'LD' then trataLeDados
            else if sCmd = 'PN' then trataPolinomio
            else if sCmd = 'HI' then trataHistograma

            else if sCmd = 'DM' then trataDominio
            else if sCmd = 'CD' then trataContraDominio

            else if sCmd = 'CC' then trataConfigCaneta
            else if sCmd = 'TB' then trataTextoBraille
            else if sCmd = 'TX' then trataTextoTinta
            else if sCmd = 'CT' then trataConfigTextoTinta

            else
                erroDetectado := true;

            if erroDetectado then
                begin
                    mostraErro;
                    break;
                end;
        end;
end;

{--------------------------------------------------------}
{                 visualizao do grfico                }
{--------------------------------------------------------}

procedure visualiza (demo: boolean);
var
    nomeArqTemp: string;
begin
    clrscr;
    gotoxy (1, 25);
    saidaGrafica := paraTela;
    criaBitmapParaDesenho;

    if demo then
        produzOGraficoDemo
    else
        produzOGrafico;

    nomeArqTemp := GetTempFile('BMP');
    FGrafico.SaveToFile(nomeArqTemp);
    FGrafico.Free;

    openBmp (nomeArqTemp);
    paintBmp (0, 0);
    readkey;
    closeBmp;
    deleteFile (nomeArqTemp);

    clrscr;
    gfmsg.tituloJanela(nomeArq);
    gotoxy (1, 3);
end;

{--------------------------------------------------------}
{               grava o arquivo JPG ou PNG               }
{--------------------------------------------------------}

procedure gravaArquivoGrafico;
var
    nomeArqGraf: string;
    c: char;
    FJpeg: TJPEGImage;
    FPNG: TPNGObject;
label fim;
begin
    textBackground (MAGENTA);
    mensagem ('GFNOMGRF', 1);   {'Informe o arquivo JPG ou PNG a gravar'}
    textBackground (BLACK);
    nomeArqGraf :=dvarq.obtemNomeArq(10);
    if nomeArqGraf = '' then
        begin
            mensagem ('GFDESIST', 1);    {'Desistiu'}
            goto fim;
        end;

    if FileExists(nomeArqGraf) then
        begin
            mensagem ('GFSOBRESC', 0);   {'Arquivo j existe, sobrescreve? '}
            if popupMenuPorLetra('SN') <> 'S' then
                begin
                    mensagem ('GFDESIST', 1);    {'Desistiu'}
                    goto fim;
                end
            else
                DeleteFile(nomeArqGraf);
        end;

    saidaGrafica := paraTela;
    criaBitmapParaDesenho;
    produzOGrafico;

    if (uppercase(copy (nomeArqGraf, length(nomeArqGraf)-3, 4)) = '.JPG') or
       (uppercase(copy (nomeArqGraf, length(nomeArqGraf)-4, 5)) = '.JPEG') then
        begin
            FJpeg := TJPEGImage.Create;
            FJpeg.Assign(FGrafico);
            FJpeg.SaveToFile(nomeArqGraf);
            FJpeg.Free;
        end
    else
    if (uppercase(copy (nomeArqGraf, length(nomeArqGraf)-3, 4)) = '.PNG') then
        begin
            FPNG := TPNGObject.Create;
            FPNG.Assign(FGrafico);
            FPNG.SaveToFile(nomeArqGraf);
            FPNG.Free;
        end
    else
    if (uppercase(copy (nomeArqGraf, length(nomeArqGraf)-3, 4)) = '.BMP') then
        FGrafico.SaveToFile(nomeArqGraf)
    else
        begin
            mensagem ('GFNAOSV', 2);  {'Arquivo grfico no foi salvo'}
            FGrafico.Free;
            goto fim;
        end;

    mensagem ('GFOK', 1);
    FGrafico.Free;

fim:
    mensagem ('GFAPTENT', 0);    {'Aperte enter...'}
    repeat
        c := readkey;
    until (c = ESC) or (c = ENTER);
    clrscr;
    tituloJanela(nomeArq);
    gotoxy (1, 3);
end;

end.

